using UnityEngine;
using System.Collections;

public class Bacteria : MonoBehaviour {

public GameObject deathParticle;
public GameObject bacMesh;
public GameObject dodgeParticles;
public GameObject awarePlane;
public GameObject extraGrowthPlane;
public int evoPoints;
public float hitPoints;
public float maxHitPoints;
public float speed;
public float scale;
public bool dodge;
public float dodgeRange;
private bool dodgeModeActive = false;
public bool playerAware;
public float awareRange;
public int growthRate;
public bool camo;
public float camoPower;

private GameObject player;
private GameControl gControl;
private bool pause = true;


public Color myColor;
public Color MyColor {
	get { return myColor; }
	set {
		if(value.r > 1) value.r = 1;
		else if(value.r < 0) value.r = 0;
		if(value.g > 1) value.g = 1;
		else if(value.g < 0) value.g = 0;
		if(value.b > 1) value.b = 1;
		else if(value.b < 0) value.b = 0;

		bacMesh.renderer.material.color = value;
		myColor = value;
	}
}

private int healthEvoCost = 3;
private int speedEvoCost = 4;
private int sizeEvoCost = 2;
private int awareEvoCost = 13;
private int awareRangeEvoCost = 4;
private int dodgeEvoCost = 16;
private int dodgeRangeEvoCost = 5;
private int growthEvoCost = 20;
private int camoEvoCost = 8;
private int camoPowerEvoCost = 3;

private Vector3 dir;

	// Use this for initialization
	void Start () {
	gControl = (GameObject.FindWithTag("GameControl")).GetComponent<GameControl>();
	dir = new Vector3(Random.Range(0f, 1f), 0, Random.Range(0f, 1f)); //Initial random direction
	dir.Normalize();
	MyColor = bacMesh.renderer.material.color;
	hitPoints = maxHitPoints;
	player = GameObject.FindWithTag("Player");
	}
	
	// Update is called once per frame
	void Update () {
		if(!pause) {
		
			
			
			
			if(playerAware) {
				float distance = Vector3.Distance(transform.position, player.transform.position);

				if(distance < awareRange) {
					dir = -(player.transform.position - transform.position);
					dir.y = 0;
					awarePlane.renderer.enabled = true;
				}

			}
			
			if(dodge && !dodgeModeActive) {
				if(testForBullets(transform.position, dodgeRange)) {
					StartCoroutine(dodgeBoost());
				}
				
			}
			
			dir+= new Vector3(Random.Range(-0.1f, 0.1f), 0, Random.Range(-0.1f, 0.1f)); //Some random movement
			dir.Normalize();
			
			
			if(Vector3.Distance(transform.position+dir, Vector3.zero) > gControl.mapRadius) { //Out of map?
				dir = dir*-1;
			}
			
			transform.Translate(dir * speed * Time.deltaTime);
		}
		pause = gControl.pause;
	}
	
	public bool testForBullets(Vector3 pos, float r) {
		Collider[] cols = Physics.OverlapSphere(pos, r);
		foreach(Collider c in cols) {
			if(c.gameObject.tag == "Bullet") {
				return true;
			}
		}
		return false;
	}
	
	IEnumerator dodgeBoost() {
		dodgeModeActive = true;
		speed += 1.5f;	
		yield return new WaitForSeconds(3.0f);
		speed -= 1.5f;
		dodgeModeActive = false;
	}
	
	public void gotHit(float dmg) {
		hitPoints -= dmg;
		if(hitPoints <= 0.0f) {
			gControl.bacteriaDied();
			Instantiate(deathParticle,transform.position, Quaternion.identity);
			AudioSource.PlayClipAtPoint(audio.clip, transform.position);
			Destroy(gameObject);
		}
		else {
			StartCoroutine(flash());
			getEvoPoints(1);
		}
	}
	
	public void Evolve() {
		
		int evolveChoice = Random.Range(0,7);
		switch(evolveChoice) {
			case 0: 
				getEvoPoints(10);
				break;
			case 1:
				if(evoPoints >= healthEvoCost) {
					useEvoPoints(healthEvoCost);
					maxHitPoints += Random.Range(0.5f, 1.5f);
					hitPoints = maxHitPoints;
					MyColor = new Color(MyColor.r+0.1f, MyColor.g-0.1f, MyColor.b-0.1f, MyColor.a);
				}
				else {
					//getEvoPoints(5);
				}
				break;
				
			case 2: 
				if(evoPoints >= speedEvoCost) {
					useEvoPoints(speedEvoCost);
					speed += Random.Range(0.1f, 1.0f);
					MyColor = new Color(MyColor.r-0.1f, MyColor.g+0.1f, MyColor.b-0.1f, MyColor.a);
				}
				else {
					//getEvoPoints(5);
				}
				break;
			
			case 3: 
				if(evoPoints >= sizeEvoCost) {
					useEvoPoints(sizeEvoCost);
					scale -= 0.05f;
					transform.localScale = new Vector3(scale, scale, scale);
					MyColor = new Color(MyColor.r-0.1f, MyColor.g-0.1f, MyColor.b+0.1f, MyColor.a);
				}
				else {
					//getEvoPoints(5);
				}
				break;
			
			case 4: 
				if(evoPoints >= growthEvoCost) {
					useEvoPoints(growthEvoCost);
					growthRate += 1;
					scale += 0.1f;
					extraGrowthPlane.renderer.enabled = true;
				}
				else {
					//getEvoPoints(5);
				}
				break;
				
			case 5:
				if(!playerAware) { //Doesn't have awareness yet
					if(evoPoints >= awareEvoCost) {
						useEvoPoints(awareEvoCost);
						playerAware = true;
						awareRange = 3.0f;
						awarePlane.renderer.enabled = true;
					}
				}
				else if(evoPoints >= awareRangeEvoCost) { //Has awareness, try increasing range
					useEvoPoints(awareRangeEvoCost);
					awareRange += Random.Range(1.0f, 3.0f);
				}
				else {
					//getEvoPoints(5);
				}
				break;
				
			case 6:
				if(!dodge) { //Doesn't have dodge yet
					if(evoPoints >= dodgeEvoCost) {
						useEvoPoints(dodgeEvoCost);
						dodge = true;
						dodgeRange = 1.0f;
						dodgeParticles.particleEmitter.emit = true;
					}
				}
				else if(evoPoints >= dodgeRangeEvoCost) { //Has dodge, try increasing range
					useEvoPoints(dodgeRangeEvoCost);
					dodgeRange += Random.Range(0.5f, 1.5f);
				}
				else {
					//getEvoPoints(5);
				}
				break;
				
			default: 
			Debug.Log("random evo out of bounds");
			break;
		}
		getEvoPoints(5);
	}

	public void useEvoPoints(int pts) {
		evoPoints -= pts;
	}
	public void getEvoPoints(int pts) {
		evoPoints += pts;
	}
	
	IEnumerator flash() {
		bacMesh.renderer.material.shader = Shader.Find("Unlit/Texture");
		bacMesh.renderer.material.color = Color.white;
		yield return new WaitForSeconds(0.05f);
		bacMesh.renderer.material.shader = Shader.Find("Diffuse");
		bacMesh.renderer.material.color = MyColor;
	}
	
}
