using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameControl : MonoBehaviour {

public float mapRadius;
public GameObject bacteria;
public int numToSpawn;
public double levelTime = 20.0d;
public double resetLevelTime = 20.0d;
public bool pause = true;
public bool doneEvolving = false;
public List<GameObject> bacteriaList = new List<GameObject>();
public GameObject player;
public Player playerScript;
private int roundNumber = 1;
public int evoPoints = 0;
private int bacteriaCount = 0;
private int gameMode = 0;

	// Use this for initialization
	void Start () {
		playerScript = player.GetComponent<Player>();
		gameMode = PlayerPrefs.GetInt("GameMode");
		StartCoroutine(timer());		
		StartCoroutine(spawnMap(20));
	}
	
	// Update is called once per frame
	void Update () {
		
		if(levelTime < 0) {
			resetLevelTime += 1d;
			levelTime = resetLevelTime;
			doneEvolving = false;			
			StartCoroutine(LevelChange());
			roundNumber++;
			
			pause = true;
		}
		if(bacteriaCount <= 0) {
			if(gameMode == 0) {
			PlayerPrefs.SetInt("RoundTotal", roundNumber);
			Application.LoadLevel("WinScene");
			}
			else if(gameMode == 1) {
				StartCoroutine(spawnMap(10));
			}
		}
	}
	
	IEnumerator spawnMap(int num) {
		for(int i=0; i<num; i++) {
			GameObject bac = (GameObject)Instantiate(bacteria, Vector3.zero, Quaternion.identity);
			setRandomPosition(bac);
			bacteriaCount++;
			yield return new WaitForSeconds(0.05f);
		}	
		doneEvolving = true;
	}
	
	public void bacteriaDied() {
		bacteriaCount--;
	}
	
	
	void setRandomPosition(GameObject bac) { //For initial setup
			Vector3 randVec = new Vector3(Random.Range(-1.0f, 1.0f),0, Random.Range(-1.0f, 1.0f));
			randVec.Normalize();
			float rangeFromCenter = Random.Range(0f,mapRadius-1.0f);
			bac.transform.Translate(randVec * rangeFromCenter);
			//Don't spawn too close to player
			if(Vector3.Distance(player.transform.position, bac.transform.position) < 3.0f) {
				setRandomPosition(bac);
			}
	}
	
	void setOffsetPosition(GameObject bac) { //When bacteria are multiplying
		Vector3 offset = new Vector3(Random.Range(-0.5f, 0.5f), 0, Random.Range(-0.5f, 0.5f));
		bac.transform.position += offset;
	}
	
	IEnumerator LevelChange() {
		evoPoints += 6;
		
		bacteriaList.Clear();
		bacteriaList.AddRange(GameObject.FindGameObjectsWithTag("Bacteria"));
		bacteriaCount = bacteriaList.Count;
		//levelTime += bacteriaCount / 4d;
	
		if(bacteriaCount > 100) {
			if(gameMode==0) {
				PlayerPrefs.SetInt("RoundTotal", roundNumber);
				Application.LoadLevel("LoseScene");
			}
		}
		else if(bacteriaCount <= 0) {
			if(gameMode==0) {
				PlayerPrefs.SetInt("RoundTotal", roundNumber);
				Application.LoadLevel("WinScene");
			}
		}
		else foreach(GameObject bac in bacteriaList) {
			if(bac != null) {
				Bacteria bScript = bac.GetComponent<Bacteria>();
				bScript.Evolve(); //Evolve old bacteria too? Hmm..
				int children = bScript.growthRate;
				
				for(int i=0; i<children; i++) {	  //Bacteria can spawn several children if growthrate has evolved
						GameObject newBac = (GameObject)Instantiate(bac, bac.transform.position, Quaternion.identity);
						setOffsetPosition(newBac);
						bScript = newBac.GetComponent<Bacteria>();
						//bScript.evoPoints = 0; //Reset child evo points?
						bScript.Evolve(); //Evolve the child					
						yield return new WaitForSeconds(0.05f);
						bacteriaCount++;
				}
			}
			
		}
		doneEvolving = true;
	}
	
	
	void OnGUI() {
		GUILayout.BeginArea(new Rect(20,20, 450, 60));
		GUILayout.BeginHorizontal();
		GUILayout.Label("Round: " + roundNumber);
		GUILayout.Label("Time: " + levelTime.ToString("f2"));
		if(gameMode == 1) {
			GUILayout.Label("FREEFORM GAMEMODE!");
		}
		GUILayout.EndHorizontal();
		GUILayout.Label("Bacteria alive: " + bacteriaCount);
		GUILayout.EndArea();
		if(pause == true) {
			GUILayout.BeginArea(new Rect(20,80,200,300));
			GUILayout.Label("Evolution points: " + evoPoints);
			
			if(GUILayout.Button("+Max bullets (9p)")) {
				if(evoPoints >= 9) {
					playerScript.maxBullets++;
					evoPoints-= 9;
				}
			}
			if(GUILayout.Button("+Speed (5p)")) {
				if(evoPoints >= 5) {
					playerScript.speed+= 0.3f;
					evoPoints-= 5;
				}
			}
			if(GUILayout.Button("+Damage (7p)")) {
				if(evoPoints >= 7) {
					playerScript.bulletDamage+= 0.3f;
					evoPoints-= 7;
				}
			}
			if(GUILayout.Button("+Bullet speed (8p)")) {
				if(evoPoints >= 8) {
					playerScript.bulletSpeed+= 0.7f;
					evoPoints-= 8;
				}
			}
			if(GUILayout.Button("+Level time (11p)")) {
				if(evoPoints >= 11) {
					resetLevelTime += 2.0d;
					levelTime += 2.0d;
					evoPoints-= 11;
				}
			}
			
			if(doneEvolving) {
			if(GUILayout.Button("Ok!")) {
				pause = false;
				StartCoroutine(timer());
			}
			}
			GUILayout.EndArea();
		}
	}
	
	IEnumerator timer() {
		while(!pause) {			
			levelTime-= 0.1d;
			yield return new WaitForSeconds(0.1f);
		}
		
	}
}
